package pouringwater_jf;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

/**
 *
 * @author Jaroslav Fibichr
 */
public class PouringWaterSolver {

    List<Instance> instances = new ArrayList<Instance>();
    String fileName;

    public PouringWaterSolver(String fileName) {
        this.fileName = fileName;
    }

    public void loadData() {
        String line;
        int num;
        FileReader fr = null;
        try {
            fr = new FileReader(fileName);
            BufferedReader buffIn = new BufferedReader(fr);

            while ((line = buffIn.readLine()) != null) {
                Instance instance = new Instance();
                StringTokenizer st = new StringTokenizer(line);

                if (!st.hasMoreTokens()) {
                    break;
                }
                instance.id = st.nextToken();
                instance.itemCount = Integer.parseInt(st.nextToken());

                // load capacities
                instance.capacities = new byte[instance.itemCount];
                for (int i = 0; i < instance.itemCount && st.hasMoreTokens(); i++) {
                    instance.capacities[i] = (byte) Integer.parseInt(st.nextToken());
                }
                // create initial state
                byte[] initial = new byte[instance.itemCount];
                for (int i = 0; i < instance.itemCount && st.hasMoreTokens(); i++) {
                    initial[i] = (byte) Integer.parseInt(st.nextToken());
                }
                instance.initialState = new State(initial, instance);
//                instance.states.add(initState);

                // create final state
                byte[] finalArray = new byte[instance.itemCount];
                for (int i = 0; i < instance.itemCount && st.hasMoreTokens(); i++) {
                    finalArray[i] = (byte) Integer.parseInt(st.nextToken());
                }
                instance.finalState = new State(finalArray, instance);
                instances.add(instance);
            }
            fr.close();
        } catch (FileNotFoundException ex) {
            System.out.println("File open error.");
        } catch (IOException ex) {
            System.out.println("File read error.");
        }
    }

    // solves the pouring water problem
    private State solve(Instance instance) {

        // parameter deteremines the chosen algorithm
        instance.states = new StateQueue(StateQueue.Method.COMB);
        instance.states.add(instance.initialState);
        StateQueue states = instance.states;

        while (states.getCount() > 0) {
            State state = states.getFirst();

            // solution found
            if (state.equals(instance.finalState)) {
                return state;
            }

            State newState;
            // expand State
            for (int i = 0; i < instance.itemCount; i++) {

                byte cap = instance.capacities[i];
                // fill bucket
                if ((newState = state.fill(i, cap)) != null) {
                    states.add(newState);
                }

                // empty bucket
                if ((newState = state.empty(i)) != null) {
                    states.add(newState);
                }

                // pour bucket to some other
                for (int j = 0; j < instance.itemCount; j++) {
                    if (j == i) {
                        continue;
                    }
                    if ((newState = state.pour2(i, instance.capacities, j)) != null) {
                        states.add(newState);
                    }
                }
            }
        }
        // no solution
        return null;
    }

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {

        // time counter
        long start;
        double elapsed = 0;

        if (args.length < 1) {
            System.out.println("Not enough parameters.");
            System.exit(0);
        }
        PouringWaterSolver solver = new PouringWaterSolver(args[0]);

        // load all instances data (whole file)
        solver.loadData();

        // run all instances
        for (int i = 0; i < solver.instances.size(); ++i) {

            Instance ins = solver.instances.get(i);

            System.out.print("Instance: ");
            System.out.print(ins.id + " (" + ins.itemCount + "): ");

            // solve
            State state = solver.solve(ins);
            if (state != null) {
                int usedStates = ins.states.getVisitedStatesCount() - ins.states.getCount();
                System.out.println("solution: \t" + usedStates + "\t" + state.steps);
//                System.out.print(" state count = " + usedStates);
//                System.out.println(", path length = " + state.steps);
//                System.out.println(", steps count: " + ins.states.stepsCount);
            } else {
                System.out.println("Solution not found.");
            }

            // error evaluation
//            double error = ((double) (solBF.value - solH.value)) / solBF.value;
//            maxError = error > maxError ? error : maxError;
//            totalError += error;
//            instCount++;


        }

    }
}
