package pouringwater_jf;

import java.util.HashSet;
import java.util.LinkedList;
import java.util.PriorityQueue;
import java.util.Queue;
import pouringwater_jf.comparator.CombCmp;
import pouringwater_jf.comparator.SumDiffCmp;
import pouringwater_jf.comparator.TotalFinalCmp;

/**
 *
 * @author yare
 */
public class StateQueue {

    public Queue<State> states;
    public HashSet<Integer> hashes = new HashSet<Integer>();
    public int stepsCount = 0;

    public static enum Method {

        BFS, TOTAL_FINAL, SUM_DIFF, COMB
    }

    public StateQueue() {
        states = new LinkedList<State>();
    }

    public StateQueue(Method method) {
        switch (method) {
            case BFS:
                states = new LinkedList<State>();
                break;
            case TOTAL_FINAL:
                states = new PriorityQueue<State>(1, new TotalFinalCmp());
                break;
            case SUM_DIFF:
                states = new PriorityQueue<State>(1, new SumDiffCmp());
                break;
            case COMB:
                states = new PriorityQueue<State>(1, new CombCmp());
                break;
            default:
                System.out.println("Method not supported.");
        }
    }

    public boolean add(State state) {

        stepsCount++;
        int hash = state.hashCode();

        if (!hashes.contains(hash)) {
            states.add(state);
            hashes.add(hash);
            return true;
        } else {
            hashes.add(hash);
            return false;
        }
    }

    public void clear() {
        states.clear();
        hashes.clear();
    }

    public int getVisitedStatesCount() {
        return hashes.size();
    }

    public int getCount() {
        return states.size();
    }

    public State getFirst() {
        return states.remove();
    }
}
